/***************************************************************************/
/***  Name         : trans.sql                                           ***/
/***                                                                     ***/
/***  Description  : Stored procedure to transfer inventory.             ***/
/***                                                                     ***/
/***  Author       : First Choice Software, Inc                          ***/
/***                 4412 Spicewood Springs Road Suite 701               ***/
/***                 Austin, TX 78759                                    ***/
/***                                                                     ***/
/***  Usage        : First Choice Confidential                           ***/
/***                                                                     ***/
/***  Preconditions: None known                                          ***/
/***                                                                     ***/
/***  Postcondition: None known                                          ***/
/***                                                                     ***/
/***  Platforms    : This version supports SQL Server version 6.0 or     ***/
/***                  later.                                             ***/
/***                                                                     ***/
/***  Copyright (C) 1997 First Choice Software, Inc.                     ***/
/***  All Rights Reserved                                                ***/
/***                                                                     ***/
/***************************************************************************/

/***************************************************************************/
/***     Header declarations.                                            ***/
/***************************************************************************/

use <db>
go

					/* Drop stored procedures, if      */
					/*  if defined                     */
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'part_transfer')
   DROP PROCEDURE part_transfer
go

					/* Drop stored procedures, if      */
					/*  if defined                     */
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'trans_internal')
   DROP PROCEDURE trans_internal
go

/***************************************************************************/
/***     Transfer part(s)                                                ***/
/***************************************************************************/

CREATE PROCEDURE trans_internal @pr_objid INT, 
                                @other_objid INT, 
                                @mod_objid INT,
                                @part_objid INT,
                                @f_bin_objid INT,
                                @f_loc_objid INT,
                                @from_good INT,
                                @t_bin_objid INT,
                                @t_loc_objid INT,
                                @to_good INT,
                                @quantity INT,
                                @serial_num VARCHAR(40),
                                @trans_type INT,
                                @user_objid INT,
                                @trans_date VARCHAR(30),
                                @ref_id VARCHAR(20),
                                @notes VARCHAR(60),
                                @gen_act INT,
                                @use_fifo INT,
                                @update_cost VARCHAR(20),
                                @trans_objid INT OUTPUT,
                                @ret_val INT OUTPUT AS
BEGIN
  DECLARE @is_serial INT,               /* Is it a serialized transaction. */
          @move_type INT,               /* Part movement type (0-3).       */
          @cost_decimal DECIMAL(19,4),  /* Cost of transfer (as a decimal  */
          @t_decimal DECIMAL(19,4),     /* Cost for detail (as a decimal). */
                                        /*  for Clarify 6.0+).             */
          @cost_real FLOAT,             /* Cost for transfer (as a real for*/
                                        /*  Clarify before 6.0).           */
          @schema_rev INT,              /* Schema revision of the DB.      */
          @mod_level VARCHAR(10),       /* Revision (mod_level).           */
          @part_num VARCHAR(30),        /* Part number.                    */
          @trans_id VARCHAR(80),        /* Transaction ID.                 */
          @pgq_from INT, @pbq_from INT, /* Part good/bad qty for from inst.*/
          @pgq_to INT, @pbq_to INT,     /* Part good/bad qty for to inst.  */
          @bin_id INT,                  /* Bin objid of part_view record.  */
          @t_bin_name VARCHAR(20),      /* Name of the "to" bin.           */
          @f_bin_name VARCHAR(20),      /* Name of the "from" bin.         */
          @t_loc_name VARCHAR(20),      /* Name of the "to" location.      */
          @f_loc_name VARCHAR(20),      /* Name of the "from" location.    */
          @add_str VARCHAR(255),        /* Additional info string.         */
          @from_class INT,              /* Inv class of from location.     */
          @to_class INT,                /* Inv class of to location.       */
          @num_fifo INT,                /* Number of FIFO units avail.     */
          @direction VARCHAR(10),       /* Direction of transfer.          */
          @left_to_cost INT,            /* Number of units left to cost.   */
          @total_cost FLOAT,            /* Amount of costing.              */
          @total_cost_dec DECIMAL(19,4),/* Amount of costing (6.0+).       */
          @qty_avail INT,               /* Number of FIFO objects at a cost*/
          @cost REAL,                   /* FIFO cost.                      */
          @date_avail DATETIME,         /* Date of FIFO transaction.       */

          @gbst_objid INT,              /* Activity string objid.          */
          @base_objid INT,              /* Base objid for all sequences.   */
          @from_inst_objid INT,         /* Objid of from part inst record. */
          @to_inst_objid INT,           /* Objid of to part inst record.   */
          @dtl_objid INT,               /* Part request objid if needed.   */
          @act_objid INT,               /* Activity log objid.             */
          @o_objid INT,                 /* Other objid for receive parts.  */
          @f_auth_objid INT,            /* From part auth level.           */
          @t_auth_objid INT,            /* To part auth level.             */
          @loc_objid INT,               /* Location FIFO is coming from.   */
          @fifo_objid INT,              /* Objid of FIFO record.           */
          @new_fifo_objid INT,          /* Objid of new built FIFO record. */
          @new_td_objid INT,            /* Objid of new trans detail rec.  */
          @qoh INT,                     /* Quantity (good) on hand for auth*/
          @part_rol INT,                /* Part reorder level for auth.    */
          @source VARCHAR(20),          /* Source of the FIFO costing rec. */
          @t_date DATETIME,             /* Transaction date in date format.*/
          @t_str VARCHAR(255)           /* Temporary string.               */

                                        /* Set serialized flag.            */
  IF @serial_num <> ''
  BEGIN
     SELECT @is_serial = 1
  END
  ELSE
  BEGIN
     SELECT @is_serial = 0
  END

                                        /* Get the activity string objid.  */
                                        /* If not found, error and leave.  */
  SELECT @gbst_objid = objid FROM table_gbst_elm WHERE rank = 12300
  IF @gbst_objid IS NULL
  BEGIN
     SELECT @ret_val = -22
     GOTO end_proc
  END

					/* Get base objid for database.    */
  SELECT @base_objid = site_id*(268435456) FROM adp_db_header

                                        /* Set the movement type.          */
  SELECT @move_type = 0
  IF @from_good = 0
  BEGIN
     SELECT @move_type = 2
  END
  IF @to_good = 0
  BEGIN
     SELECT @move_type = @move_type + 1
  END

                                        /* Get the revision level of DB.   */
  SELECT @schema_rev = schema_rev FROM adp_db_header

                                        /* Get some needed data.           */
  SELECT @mod_level = mod_level FROM table_mod_level WHERE objid = @mod_objid
  SELECT @part_num = part_number FROM table_part_num WHERE objid = @part_objid
  SELECT @t_bin_name = bin_name FROM table_inv_bin WHERE objid = @t_bin_objid
  SELECT @f_bin_name = bin_name FROM table_inv_bin WHERE objid = @f_bin_objid
  SELECT @t_loc_name = location_name, @to_class = inv_class
    FROM table_inv_locatn WHERE objid = @t_loc_objid
  SELECT @f_loc_name = location_name, @from_class = inv_class
    FROM table_inv_locatn WHERE objid = @f_loc_objid

                                        /* Figure out direction: IN, OUT   */
                                        /*  or TRAN. Tran = transfer among */
                                        /*  inv_locations. IN is from an   */
                                        /*  expense GL and OUT is to an exp*/
  IF @from_class <> 2
  BEGIN
    SELECT @loc_objid = @f_loc_objid
    IF @to_class = 2
    BEGIN
      SELECT @direction = 'OUT'
    END
    ELSE
    BEGIN
      SELECT @direction = 'TRAN'
    END
  END
  ELSE 
  BEGIN
    SELECT @loc_objid = @t_loc_objid
    SELECT @direction = 'IN'
  END

                                        /* If they don't use FIFO cost,    */
                                        /*  then just find costs from      */
                                        /*  normal means.                  */
  IF @use_fifo = 0
  BEGIN
     GOTO find_normal_cost
  END

                                        /* If they supply the unit cost... */
                                        /* (Work order only for now).      */
                                        /* Build up the proper cost.       */
  IF @use_fifo = 2
  BEGIN
     IF @schema_rev >= 45
     BEGIN
       SELECT @cost_decimal = CONVERT(DECIMAL(19,4), @update_cost) * @quantity
     END
     ELSE
     BEGIN
       SELECT @cost_real = CONVERT(FLOAT, @update_cost) * @quantity
     END                                   
     GOTO check_auths
  END

                                        /* If we're using FIFO...          */
                                        /* Find out how many units avail.  */
                                        /* If none (or not enough), error. */
  SELECT @num_fifo = sum(qty_avail) FROM table_fifo_cost
    WHERE fifo_cost2inv_locatn = @loc_objid AND 
	  fifo_cost2part_number = @part_objid
  IF @num_fifo IS NULL
  BEGIN
not_enough_fifo:
    SELECT @ret_val = -24
    GOTO end_proc
  END 
  IF @num_fifo < @quantity AND @direction <> 'IN'
  BEGIN
     GOTO not_enough_fifo
  END

                                        /* Get necessary objids.           */
                                        /* Need it now.                    */
  EXEC csp_new_oid 244, @trans_objid OUT
  SELECT @trans_objid = @trans_objid + @base_objid

                                        /* Remember how many units they    */
                                        /*  are moving. Start with no cost.*/
					/* While there's more to move...   */
  SELECT @left_to_cost = @quantity
  SELECT @total_cost = 0
  SELECT @total_cost_dec = 0
  WHILE @left_to_cost > 0
  BEGIN
                                        /* Now we need a cursor to look at */
                                        /*  the fifo_cost records in order */
                                        /*  and to consume the units as    */
                                        /*  needed.                        */
                                        /* For out or transfer, use FIFO.  */
                                        /* If coming in, we're adding to   */
                                        /*  the most recent cost (LIFO).   */
    IF @direction <> 'IN'
    BEGIN
      DECLARE fifo_cursor CURSOR FOR 
        SELECT objid, qty_avail, cost, source, date_avail
          FROM table_fifo_cost
          WHERE fifo_cost2inv_locatn = @loc_objid AND fifo_cost2part_number = @part_objid
          ORDER BY date_avail
    END
    ELSE
    BEGIN
      DECLARE fifo_cursor CURSOR FOR 
        SELECT objid, qty_avail, cost, source, date_avail
          FROM table_fifo_cost
          WHERE fifo_cost2inv_locatn = @loc_objid AND fifo_cost2part_number = @part_objid
          ORDER BY date_avail desc
    END
    
					/* Open the cursor. Get the next   */
					/*  fifo cost.                     */
					/* If inbound, add qty to the fifo */
					/*  cost record and transact all.  */
                                        /* Get total cost for transaction. */
    OPEN fifo_cursor
/* Use the next line with Clarify 4.x and 5.x                              */
/*    FETCH NEXT FROM fifo_cursor INTO @fifo_objid, @qty_avail, @cost, @source, @date_avail */

/* Use the next line with Clarify 6.x+                                     */
    FETCH NEXT FROM fifo_cursor INTO @fifo_objid, @qty_avail, @cost_decimal, @source, @date_avail
    
    WHILE (@@FETCH_STATUS = 0)
    BEGIN
/* Use the next line with Clarify 6.x+                                     */
      SELECT @cost = CONVERT(FLOAT, @cost_decimal)

      IF @direction = 'IN'
      BEGIN
         SELECT @total_cost = @cost * @quantity

                                        /* Add the trans_fifo_dtl for this */
                                        /*  "IN" record.                   */
      	 EXEC csp_new_oid 3540, @new_td_objid OUT
	 SELECT @new_td_objid = @new_td_objid + @base_objid
         SELECT @t_date = getdate()
         SELECT @t_decimal = CONVERT(DECIMAL(19,4), @total_cost)
	 INSERT INTO table_trans_fifo_dtl (objid, part_number, date_avail, quantity, 
	        source, cost, trans_fifo_dtl2part_trans) VALUES
		(@new_td_objid, @part_num, @t_date, @left_to_cost, @source, 
		@t_decimal, @trans_objid)

	 UPDATE table_fifo_cost SET qty_avail = qty_avail + @left_to_cost
	   WHERE objid = @fifo_objid         
         SELECT @left_to_cost = 0
	 GOTO end_cursor
      END         

					/* If there's enough in this fifo  */
					/*  record to handle it all, update*/
					/*  the fifo cost record and trans */
					/*  act all of it.                 */
					/* Else, delete the fifo cost rec  */
					/*  and only transact that amount. */
                                        /* Update total cost for these the */
                                        /*  parts being used.              */
      IF @qty_avail > @left_to_cost
      BEGIN
      	 EXEC csp_new_oid 3540, @new_td_objid OUT
	 SELECT @new_td_objid = @new_td_objid + @base_objid
         SELECT @t_date = getdate()
         SELECT @t_decimal = CONVERT(DECIMAL(19,4), @cost) * @left_to_cost
	 INSERT INTO table_trans_fifo_dtl (objid, part_number, date_avail, quantity, 
	        source, cost, trans_fifo_dtl2part_trans) VALUES
		(@new_td_objid, @part_num, @t_date, @left_to_cost, @source, 
		@t_decimal, @trans_objid)

	 UPDATE table_fifo_cost SET qty_avail = qty_avail - @left_to_cost
	   WHERE objid = @fifo_objid         
         SELECT @total_cost = @total_cost + @cost * @left_to_cost
         SELECT @qty_avail = @left_to_cost
         SELECT @left_to_cost = 0
      END
      ELSE
      BEGIN
      	 EXEC csp_new_oid 3540, @new_td_objid OUT
	 SELECT @new_td_objid = @new_td_objid + @base_objid
         SELECT @t_date = getdate()
         SELECT @t_decimal = CONVERT(DECIMAL(19,4), @cost) * @qty_avail
	 INSERT INTO table_trans_fifo_dtl (objid, part_number, date_avail, quantity, 
	        source, cost, trans_fifo_dtl2part_trans) VALUES
		(@new_td_objid, @part_num, @t_date, @qty_avail, @source, 
		@t_decimal, @trans_objid)

	 DELETE FROM table_fifo_cost WHERE objid = @fifo_objid          
         SELECT @total_cost = @total_cost + @cost * @qty_avail
	 SELECT @left_to_cost = @left_to_cost - @qty_avail
      END

					/* If this is a transfer, build    */
					/*  a new fifo cost rec for the    */
					/*  inventory moved in.            */
      IF @direction = 'TRAN'
      BEGIN
	 EXEC csp_new_oid 3532, @new_fifo_objid OUT
	 SELECT @new_fifo_objid = @new_fifo_objid + @base_objid
	 INSERT INTO table_fifo_cost (objid, date_avail, qty_avail, cost, source,
		fifo_cost2part_number, fifo_cost2inv_locatn) VALUES
		(@new_fifo_objid, @date_avail, @qty_avail, @cost, @source,
		 @part_objid, @t_loc_objid)        
      END

      IF @left_to_cost = 0
      BEGIN
         GOTO end_cursor
      END

                                        /* Get next FIFO cursor object     */
/* Use the next line with Clarify 4.x and 5.x                              */
/*    FETCH NEXT FROM fifo_cursor INTO @fifo_objid, @qty_avail, @cost, @source, @date_avail */

/* Use the next line with Clarify 6.x+                                     */
    FETCH NEXT FROM fifo_cursor INTO @fifo_objid, @qty_avail, @cost_decimal, @source, @date_avail
    END
end_cursor:
    DEALLOCATE fifo_cursor
  END

                                        /* Change to the correct format    */
                                        /*  based on Clarify version.      */
                                        /* Don't need to figure out the    */
                                        /*  standard cost.                 */
  IF @schema_rev >= 45
  BEGIN
    SELECT @cost_decimal = CONVERT(DECIMAL(19,4), @total_cost)
  END
  ELSE
  BEGIN
    SELECT @cost_real = @total_cost
  END                                   
  GOTO check_auths                                     

                                        /* Get the price of the part/rev.  */
                                        /* If no std cost record, price=0. */
                                        /* We need to get it one of two    */
                                        /*  ways (decimal/float).          */
find_normal_cost:
  IF @schema_rev >= 45
  BEGIN
     SELECT @cost_decimal = price FROM table_price_num
       WHERE type = 0 AND part_mod = @mod_objid AND eff_date <= @trans_date
             AND exp_date >= @trans_date
     IF @cost_decimal IS NULL
     BEGIN
        SELECT @cost_decimal = 0
     END
     SELECT @cost_decimal = @cost_decimal * @quantity
  END
  ELSE
  BEGIN
     SELECT @cost_real = price FROM table_price_num
       WHERE type = 0 AND part_mod = @mod_objid AND eff_date <= @trans_date
             AND exp_date >= @trans_date
     IF @cost_real IS NULL
     BEGIN
        SELECT @cost_real = 0
     END
     SELECT @cost_real = @cost_real * @quantity
  END

check_auths:
                                        /* Get the objids for the part auth*/
                                        /*  records in case we need to go  */
                                        /*  later and modify them (and the */
                                        /*  auth levels.                   */
  SELECT @f_auth_objid = objid FROM table_part_auth 
    WHERE part_auth2inv_bin = @f_bin_objid AND n_auth_parts2part_mod = @mod_objid
  SELECT @t_auth_objid = objid FROM table_part_auth 
    WHERE part_auth2inv_bin = @t_bin_objid AND n_auth_parts2part_mod = @mod_objid

                                        /* First, set a relation if needed.*/
  IF @trans_type = 2
  BEGIN
     SELECT @o_objid = @other_objid
     IF @pr_objid IS NOT NULL
     BEGIN
        SELECT @dtl_objid = @pr_objid
     END
     ELSE
     BEGIN
        SELECT @dtl_objid = NULL
     END
  END
  ELSE
  BEGIN
     SELECT @o_objid = NULL
  END

                                        /* If serialized, get the one price*/
                                        /*  instance. If not found, build  */
                                        /*  it.                            */
  IF @is_serial = 1
  BEGIN
    SELECT @from_inst_objid = objid, @bin_id = bin_id FROM table_parts_view
      WHERE part_num_id = @mod_objid AND part_serial_no = @serial_num
    IF @from_inst_objid IS NULL
    BEGIN
       EXEC csp_new_oid 239, @from_inst_objid OUT
       SELECT @from_inst_objid = @from_inst_objid + @base_objid
       INSERT INTO table_part_inst (objid, part_good_qty, part_bad_qty, 
               part_serial_no, part_mod, part_bin, last_pi_date, pi_tag_no, 
               last_cycle_ct, next_cycle_ct, last_mod_time,  
               last_trans_time, transaction_id, date_in_serv, warr_end_date, 
               repair_date, part_status, pick_request, good_res_qty, 
               bad_res_qty, n_part_inst2part_mod) VALUES (@from_inst_objid, 
               0, 0, @serial_num, @mod_level, '', '', '', '', '', '', '', '',
               @trans_date, '', '', '', '', 0, 0, @mod_objid)
    END
    ELSE
    BEGIN
                                        /* Else, the part_inst already     */
                                        /*  exists. Make sure it's in the  */
                                        /*  proper bin already. If not, it */
                                        /*  is an error. Leave.            */
       IF @bin_id <> @f_bin_objid
       BEGIN
          SELECT @ret_val = -20
          GOTO end_proc
       END
    END

                                        /* Build the part transaction ID.  */
                                        /* If a problem, error and leave.  */
    EXEC build_id_number 'Part Transaction ID', @trans_id OUT, @ret_val
    IF @ret_val < 0
    BEGIN
       SELECT @ret_val = -23
       GOTO end_proc
    END

                                        /* Set the part good/bad quantities*/
    IF @to_good = 1
    BEGIN
       SELECT @pgq_from = 1
       SELECT @pbq_from = 0
    END
    ELSE
    BEGIN
       SELECT @pgq_from = 0
       SELECT @pbq_from = 1
    END

    IF @gen_act = 1
    BEGIN
       EXEC csp_new_oid 21, @act_objid OUT
       SELECT @act_objid = @act_objid + @base_objid
       SELECT @t_str = CONVERT(CHAR, @quantity)
       SELECT @add_str = 'Reference: ' + @ref_id + ', From Location: ' +
              @f_loc_name + ', From Bin: ' + @f_bin_name + ', To Location: ' +
              @t_loc_name + ', To Bin: ' + @t_bin_name + ', Qty: ' + @t_str +
              ', Comments: ' + @notes
    END

                                        /* Begin transaction.              */
    BEGIN TRAN

                                        /* Update the part_inst record.    */
      UPDATE table_part_inst SET part_good_qty = @pgq_from, part_bad_qty = @pbq_from,
               part_bin = @t_bin_name, last_mod_time = @trans_date, 
               last_trans_time = @trans_date,
               transaction_id = @trans_id, part_inst2inv_bin = @t_bin_objid
        WHERE objid = @from_inst_objid
      IF @dtl_objid IS NOT NULL
      BEGIN
         UPDATE table_demand_dtl SET demand_dtl2part_inst = @from_inst_objid
           WHERE objid = @dtl_objid
      END

                                        /* Now write the part_trans record.*/
                                        /* Two versions, for float/decimal.*/
     IF @schema_rev >= 45
     BEGIN
        INSERT INTO table_part_trans (objid, transaction_id, creation_date, 
                reference_no, part_number, quantity, movement_type, notes,
                standard_cost, trans_type, ROL_ind, open_ord_ind, arch_ind,
                part_trans_owner2user, from_bin2inv_bin, to_bin2inv_bin,
                from_inst2part_inst, to_inst2part_inst, part_trans2demand_dtl, 
                recv_trans2recv_parts) VALUES
                (@trans_objid, @trans_id, @trans_date, @ref_id, @part_num, 1,
                @move_type, @notes, @cost_decimal, @trans_type, 0, 0, 0, 
                @user_objid, @f_bin_objid, @t_bin_objid, @from_inst_objid, 
                @from_inst_objid, @dtl_objid, @o_objid)
     END
     ELSE
     BEGIN
        INSERT INTO table_part_trans (objid, transaction_id, creation_date, 
                reference_no, part_number, quantity, movement_type, notes,
                standard_cost, trans_type, ROL_ind, open_ord_ind, arch_ind,
                part_trans_owner2user, from_bin2inv_bin, to_bin2inv_bin,
                from_inst2part_inst, to_inst2part_inst, part_trans2demand_dtl, 
                recv_trans2recv_parts) VALUES
                (@trans_objid, @trans_id, @trans_date, @ref_id, @part_num, 1,
                @move_type, @notes, @cost_real, @trans_type, 0, 0, 0, 
                @user_objid, @f_bin_objid, @t_bin_objid, @from_inst_objid, 
                @from_inst_objid, @dtl_objid, @o_objid)
     END
  END
  ELSE
  BEGIN
                                        /* Get the from part inst.         */
                                        /* If not there, and is an inv loc */
                                        /*  that's an error. Leave.        */
                                        /* If not there and is EXP GL,     */
                                        /*  build it. If it is there, make */
                                        /*  sure we're not making the quant*/
                                        /*  go negative. If so, error/exit.*/
    SELECT @from_inst_objid = objid, @pgq_from = part_good_qty, 
           @pbq_from = part_bad_qty
      FROM table_parts_view
      WHERE part_num_id = @mod_objid AND bin_id = @f_bin_objid
    IF @from_inst_objid IS NULL
    BEGIN
       IF @from_class = 0
       BEGIN
          SELECT @ret_val = -21
          GOTO end_proc
       END
       ELSE
       BEGIN
          EXEC csp_new_oid 239, @from_inst_objid OUT
          SELECT @from_inst_objid = @from_inst_objid + @base_objid
          INSERT INTO table_part_inst (objid, part_good_qty, part_bad_qty, 
                  part_serial_no, part_mod, part_bin, last_pi_date, pi_tag_no, 
                  last_cycle_ct, next_cycle_ct, last_mod_time,  
                  last_trans_time, transaction_id, date_in_serv, warr_end_date, 
                  repair_date, part_status, pick_request, good_res_qty, 
                  bad_res_qty, n_part_inst2part_mod, part_inst2inv_bin) 
                  VALUES (@from_inst_objid, 
                  0, 0, '', @mod_level, @f_bin_name, '', '', '', '', '', '', '',
                  @trans_date, '', '', '', '', 0, 0, @mod_objid, @f_bin_objid)
          SELECT @pgq_from = 0
          SELECT @pbq_from = 0
       END
    END
    ELSE
    BEGIN
      IF @from_class = 0
      BEGIN
        IF (@from_good = 1 AND @quantity > @pgq_from) OR 
           (@from_good = 0 AND @quantity > @pbq_from)
        BEGIN
           SELECT @ret_val = -21
           GOTO end_proc           
        END
      END
    END    

                                        /* If there's no "to" part_inst... */
                                        /*  then build it.                 */
    SELECT @to_inst_objid = objid, @pgq_to = part_good_qty, @pbq_to = part_bad_qty
      FROM table_parts_view
      WHERE part_num_id = @mod_objid AND bin_id = @t_bin_objid
    IF @to_inst_objid IS NULL
    BEGIN    
       EXEC csp_new_oid 239, @to_inst_objid OUT
       SELECT @to_inst_objid = @to_inst_objid + @base_objid
       INSERT INTO table_part_inst (objid, part_good_qty, part_bad_qty, 
               part_serial_no, part_mod, part_bin, last_pi_date, pi_tag_no, 
               last_cycle_ct, next_cycle_ct, last_mod_time,  
               last_trans_time, transaction_id, date_in_serv, warr_end_date, 
               repair_date, part_status, pick_request, good_res_qty, 
               bad_res_qty, n_part_inst2part_mod, part_inst2inv_bin) 
               VALUES (@to_inst_objid, 
               0, 0, '', @mod_level, @t_bin_name, '', '', '', '', '', '', '',
               @trans_date, '', '', '', '', 0, 0, @mod_objid, @t_bin_objid)
       SELECT @pgq_to = 0
       SELECT @pbq_to = 0
    END

                                        /* Build the part transaction ID.  */
                                        /* If a problem, error and leave.  */
    EXEC build_id_number 'Part Transaction ID', @trans_id OUT, @ret_val
    IF @ret_val < 0
    BEGIN
       SELECT @ret_val = -23
       GOTO end_proc
    END

                                        /* Adjust totals from transaction. */

    IF @from_good = 1
    BEGIN
       SELECT @pgq_from = @pgq_from - @quantity
    END
    ELSE
    BEGIN
       SELECT @pbq_from = @pbq_from - @quantity
    END
    IF @to_good = 1
    BEGIN
       SELECT @pgq_to = @pgq_to + @quantity
    END
    ELSE
    BEGIN
       SELECT @pbq_to = @pbq_to + @quantity
    END

    IF @gen_act = 1
    BEGIN
       EXEC csp_new_oid 21, @act_objid OUT
       SELECT @act_objid = @act_objid + @base_objid
       SELECT @t_str = CONVERT(CHAR, @quantity)
       SELECT @add_str = 'Reference: ' + @ref_id + ', From Location: ' +
              @f_loc_name + ', From Bin: ' + @f_bin_name + ', To Location: ' +
              @t_loc_name + ', To Bin: ' + @t_bin_name + ', Qty: ' + @t_str +
              ', Comments: ' + @notes
    END

                                        /* Start the transaction.          */
                                        /* Update the from inst record.    */
                                        /* Update the to inst record.      */
    BEGIN TRAN
      UPDATE table_part_inst SET part_good_qty = @pgq_from, 
             part_bad_qty = @pbq_from, last_mod_time = @trans_date,
             last_trans_time = @trans_date, transaction_id = @trans_id
        WHERE objid = @from_inst_objid
      UPDATE table_part_inst SET part_good_qty = @pgq_to, 
             part_bad_qty = @pbq_to, last_mod_time = @trans_date,
             last_trans_time = @trans_date, transaction_id = @trans_id
        WHERE objid = @to_inst_objid
      IF @dtl_objid IS NOT NULL
      BEGIN
         UPDATE table_demand_dtl SET demand_dtl2part_inst = @from_inst_objid
           WHERE objid = @dtl_objid
      END

                                        /* Now write the part_trans record.*/
                                        /* Two versions, for float/decimal.*/
     IF @schema_rev >= 45
     BEGIN
        INSERT INTO table_part_trans (objid, transaction_id, creation_date, 
                reference_no, part_number, quantity, movement_type, notes,
                standard_cost, trans_type, ROL_ind, open_ord_ind, arch_ind,
                part_trans_owner2user, from_bin2inv_bin, to_bin2inv_bin,
                from_inst2part_inst, to_inst2part_inst, part_trans2demand_dtl, 
                recv_trans2recv_parts) VALUES
                (@trans_objid, @trans_id, @trans_date, @ref_id, @part_num, @quantity,
                @move_type, @notes, @cost_decimal, @trans_type, 0, 0, 0, 
                @user_objid, @f_bin_objid, @t_bin_objid, @from_inst_objid, 
                @to_inst_objid, @dtl_objid, @o_objid)
     END
     ELSE
     BEGIN
        INSERT INTO table_part_trans (objid, transaction_id, creation_date, 
                reference_no, part_number, quantity, movement_type, notes,
                standard_cost, trans_type, ROL_ind, open_ord_ind, arch_ind,
                part_trans_owner2user, from_bin2inv_bin, to_bin2inv_bin,
                from_inst2part_inst, to_inst2part_inst, part_trans2demand_dtl, 
                recv_trans2recv_parts) VALUES
                (@trans_objid, @trans_id, @trans_date, @ref_id, @part_num, @quantity,
                @move_type, @notes, @cost_real, @trans_type, 0, 0, 0, 
                @user_objid, @f_bin_objid, @t_bin_objid, @from_inst_objid, 
                @to_inst_objid, @dtl_objid, @o_objid)
     END
  END

                                        /* Now we must do the part auth.   */
                                        /* Do it for from/to and good/bad  */
                                        /*  parts qty.                     */
  IF @f_auth_objid IS NOT NULL
  BEGIN
     IF @from_good = 1
     BEGIN
        SELECT @qoh = part_good_qoh, @part_rol = part_rol 
          FROM table_part_auth WHERE objid = @f_auth_objid
        IF @qoh - @quantity <= @part_rol
        BEGIN
          UPDATE table_part_trans SET ROL_ind = 1 WHERE objid = @trans_objid
        END
        UPDATE table_part_auth SET part_good_qoh = part_good_qoh - @quantity
          WHERE objid = @f_auth_objid
        
     END
     ELSE
     BEGIN
        UPDATE table_part_auth SET part_bad_qoh = part_bad_qoh - @quantity
          WHERE objid = @f_auth_objid
     END
  END
  IF @t_auth_objid IS NOT NULL
  BEGIN
     IF @to_good = 1
     BEGIN
       UPDATE table_part_auth SET part_good_qoh = part_good_qoh + @quantity
          WHERE objid = @t_auth_objid
     END
     ELSE
     BEGIN
        UPDATE table_part_auth SET part_bad_qoh = part_bad_qoh + @quantity
          WHERE objid = @t_auth_objid
     END
  END

                                        /* If they wanted an act_entry...  */
                                        /*  build it!!                     */
  IF @gen_act = 1
  BEGIN
     INSERT INTO table_act_entry (objid, act_code, entry_time, addnl_info, proxy,
       removed, entry_name2gbst_elm, act_entry2user, act_entry2part_trans) 
       VALUES (@act_objid, 12300, @trans_date, @add_str, '', 0, @gbst_objid, 
               @user_objid, @trans_objid)
  END

                                        /* Commit trans - close changes.   */
  COMMIT TRAN

                                        /* Everything was OK!!             */
  SELECT @ret_val = 0
end_proc:
END
go

/***************************************************************************/
/***     Transfer part(s)                                                ***/
/***************************************************************************/

CREATE PROCEDURE part_transfer
		 @from_loc VARCHAR(20), 
		 @from_bin VARCHAR(20),
		 @from_good INT,
		 @to_loc VARCHAR(20),
		 @to_bin VARCHAR(20),
		 @to_good INT,
		 @part_num VARCHAR(30),
		 @mod_level VARCHAR(10),
		 @domain VARCHAR(40),
		 @quantity INT,
                 @serial_num VARCHAR(40),
		 @user_name VARCHAR(30),
		 @ref_id VARCHAR(20),
		 @notes VARCHAR(60),
                 @trans_date VARCHAR(30),
		 @wo_num VARCHAR(80),   /* Not currently used.             */
		 @use_fifo INT,
		 @update_cost VARCHAR(20),
                 @gen_tb INT,
		 @ret_val INT OUTPUT AS
BEGIN
  DECLARE @t_date DATETIME,             /* Transaction date in date format.*/
          @past_time DATETIME,          /* A time in the past for time_bomb*/
          @sn_track INT,                /* Is it a serialized part?        */

          @part_objid INT,              /* Part number objid.              */
          @mod_objid INT,               /* Part revision objid.            */
          @f_loc_objid INT,             /* From location objid.            */
          @f_bin_objid INT,             /* From bin objid.                 */
          @t_loc_objid INT,             /* To location objid.              */
          @t_bin_objid INT,             /* To bin objid.                   */
          @user_objid INT,              /* Objid of user record.           */
          @trans_objid INT,             /* Part transaction objid created. */
          @emp_objid INT,               /* Employee's objid.               */
          @base_objid INT,              /* Base objid for all sequences.   */
          @tb_objid INT                 /* Objid of time bomb object.      */

					/* Get current date/time.          */
					/*  Only if one isn't supplied.    */
					/* Update the container record.    */
  IF LTRIM(RTRIM(@trans_date)) = '' OR LTRIM(RTRIM(@trans_date)) IS NULL
  BEGIN
     SELECT @t_date = getdate()
  END
  ELSE
  BEGIN
     SELECT @t_date = CONVERT(datetime, @trans_date)
  END
  SELECT @past_time = DATEADD(mm, -1, @t_date)

					/* If no user name specified, find */
					/*  one in the config item. If that*/
					/*  isn't found, use default.      */
  IF @user_name = ''
  BEGIN
     SELECT @user_name = str_value FROM table_config_itm 
      WHERE name = 'Default Logistics User'
     IF @user_name IS NULL OR @user_name = ''
     BEGIN
	SELECT @user_name = 'ClearLogistics'
     END
  END

                                        /* Can't transfer from somewhere   */
                                        /*  to itself.                     */
  IF @from_bin = @to_bin AND @from_loc = @to_loc AND @from_good = @to_good
  BEGIN
     SELECT @ret_val = -1
     GOTO end_proc
  END

                                        /* Can't transfer < 1 units.       */
  IF @quantity < 1
  BEGIN
     SELECT @ret_val = -2
     GOTO end_proc
  END

                                        /* If quantity > 1, can't have a   */
                                        /*  serial number.                 */
  IF @quantity > 1 AND @serial_num <> ''
  BEGIN
     SELECT @ret_val = -3
     GOTO end_proc
  END

                                        /* Get SN Track from part number.  */
                                        /* If part not found, error/exit.  */
  SELECT @sn_track = sn_track, @part_objid = objid
    FROM table_part_num 
    WHERE part_number = @part_num AND domain = @domain
  IF @part_objid IS NULL
  BEGIN
     SELECT @ret_val = -4
     GOTO end_proc
  END

                                        /* If the part is serialized, and  */
                                        /*  no serial number, error.       */
  IF @sn_track = 1 AND @serial_num = ''
  BEGIN
     SELECT @ret_val = -5
     GOTO end_proc
  END

                                        /* Cannot find the part revision.  */
  SELECT @mod_objid = objid FROM table_mod_level
    WHERE @mod_level = mod_level AND part_info2part_num = @part_objid
  IF @mod_objid IS NULL
  BEGIN
     SELECT @ret_val = -6
     GOTO end_proc
  END

                                        /* Make sure from location is OK.  */
  SELECT @f_loc_objid = objid FROM table_inv_locatn
    WHERE location_name = @from_loc AND active = 1 AND inv_class <> 1
  IF @f_loc_objid IS NULL
  BEGIN
     SELECT @ret_val = -7
     GOTO end_proc
  END

                                        /* Make sure from bin is OK.       */
  SELECT @f_bin_objid = objid FROM table_inv_bin
    WHERE bin_name = @from_bin AND active = 1 AND 
          inv_bin2inv_locatn = @f_loc_objid
  IF @f_bin_objid IS NULL
  BEGIN
     SELECT @ret_val = -8
     GOTO end_proc
  END

                                        /* Make sure to location is OK.    */
  SELECT @t_loc_objid = objid FROM table_inv_locatn
    WHERE location_name = @to_loc AND active = 1 AND inv_class <> 1
  IF @t_loc_objid IS NULL
  BEGIN
     SELECT @ret_val = -9
     GOTO end_proc
  END

                                        /* Make sure to bin is OK.         */
  SELECT @t_bin_objid = objid FROM table_inv_bin
    WHERE bin_name = @to_bin AND active = 1 AND 
          inv_bin2inv_locatn = @t_loc_objid
  IF @t_bin_objid IS NULL
  BEGIN
     SELECT @ret_val = -10
     GOTO end_proc
  END

                                        /* Make sure that user is OK.      */
                                        /* Get emp objid for time bomb.    */
  SELECT @user_objid = objid FROM table_user WHERE login_name = @user_name
  IF @user_objid IS NULL
  BEGIN
     SELECT @ret_val = -11
     GOTO end_proc
  END
  SELECT @emp_objid = objid FROM table_employee WHERE employee2user = @user_objid

                                        /* Call on internal version of part*/
                                        /*  transfer. If error, subtract   */
                                        /*  100 to error number. Exit.     */  
  EXEC trans_internal 0, 0, @mod_objid, @part_objid, @f_bin_objid, @f_loc_objid,
                      @from_good, @t_bin_objid, @t_loc_objid, @to_good, @quantity,
                      @serial_num, 4, @user_objid, @t_date, @ref_id, @notes,
                      1, @use_fifo, @update_cost, @trans_objid OUT, @ret_val OUT
  IF @ret_val < 0
  BEGIN
     SELECT @ret_val = @ret_val - 100
     GOTO end_proc
  END

  IF @gen_tb = 1
  BEGIN
					/* Get base objid for database.    */
					/* Get time bomb's objid.          */
    SELECT @base_objid = site_id*(268435456) FROM adp_db_header
    EXEC csp_new_oid 77, @tb_objid OUT
    SELECT @tb_objid = @tb_objid + @base_objid

                                        /* Add the time bomb.              */    
    INSERT INTO table_time_bomb (objid, title, escalate_time, end_time, 
       focus_lowid, focus_type, suppl_info, time_period, flags, left_repeat,
       report_title, property_set, users, cmit_creator2employee) VALUES
       (@tb_objid, '', @past_time, @t_date, @trans_objid, 244, '', 0,
       2293762, 0, '', '', '', @emp_objid)
  END
  
                                        /* Everything was OK!!             */
  SELECT @ret_val = 0
end_proc:
END
go

/***************************************************************************/
/***     Permission granting statements.                                 ***/
/***************************************************************************/

GRANT EXECUTE ON part_transfer TO PUBLIC
go

GRANT EXECUTE ON trans_internal TO PUBLIC
go

/***************************************************************************/
/***     Test routine.                                                   ***/
/***************************************************************************/

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'test')
   DROP PROCEDURE test
go

CREATE PROCEDURE test AS
BEGIN
  DECLARE @ret_val INT,
          @t_str VARCHAR(100)

  EXEC part_transfer 'Austin', '', 1, 'EXPGL', '', 1, 'Notebook',
            '', 'Quantity', 10, '', 'sa', 'Ref ID', 'Notes', '', '', 1, '',
            1, @ret_val OUT
/*  EXEC part_transfer 'EXPGL', '', 1, 'Austin', 'Bin 2', 1, 'Manual',
            '', 'Quantity', 3, '', 'sa', 'Ref ID', 'Notes', '', '', 2, '2.31',
            1, @ret_val OUT */
  SELECT @t_str = "Return code: " + CONVERT(CHAR, @ret_val)
  PRINT @t_str
END
go

GRANT EXECUTE ON test TO PUBLIC
go

quit
